function CLT_Random(samp_size, num_samp, statistic)
%************************************************************************/
%* The Central Limit Theorem - A simulation created to help visualize   */
%* the sampling distribution of a statistic of interested calculated    */
%* from an arbitrary population distribution.                           */
%*                                                                      */
%* Please acknowledge the program author on any publication of          */
%* scientific results based in part on use of the program and           */
%* cite the following article in which the program was described.       */
%*                                                                      */
%* E. Ruggieri, "Visualizing the Central Limit Theorem through          */
%* Simulation"                                                          */
%*                                                                      */
%* Program Author: Eric Ruggieri                                        */
%* College of the Holy Cross                                            */
%* Worcester, MA 01610                                                  */
%* Email:  eruggier@holycross.edu                                       */
%*                                                                      */
%* Copyright (C) 2014  College of the Holy Cross                        */
%*                                                                      */
%* The Central Limit Theorem simulation is free software:               */
%* you can redistribute it and/or modify it under the terms of the GNU  */
%* General Public License as published by the Free Software Foundation, */
%* either version 3 of the License, or (at your option) any later       */
%* version.                                                             */
%*                                                                      */
%* The Central Limit Theorem simulation is distributed in the           */
%* hope that it will be useful, but WITHOUT ANY WARRANTY; without even  */ 
%* the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR  */
%* PURPOSE.  See the GNU General Public License for more details.       */
%*                                                                      */
%* You should have received a copy of the GNU General Public License    */
%* along with the Central Limit Theorem simulation.  If not,            */
%* see <http://www.gnu.org/licenses/> or write to the                   */
%* Free Software Foundation, Inc.                                       */
%* 51 Franklin Street Fifth Floor                                       */
%* Boston, MA 02110-1301, USA.                                          */
%************************************************************************/
%
% Outline of the Central Limit Theorem simulation:
%   1. Use a modified version of Matlab's ginput function to capture each
%      student's population distribution.
%   2. Use a rejection sampler to draw samples from the population
%   3. Plot a histogram of the sampling distribution for the statistic of
%      interest
%
%
% Syntax for simulation:
%
% CLT_Random(samp_size, num_samp, statistic);
%
% samp_size = sample size from the population, default is 50
% num_samp = number of samples drawn from the population, default is 500
% statistic = A string indicating the statistic whose sampling distribution
%     you would like to visualize.  Possible values are: 'mean', 'median', and 'stdev'

%clear;
close all;
clc;

% Axis bounds for plot of the population distribution.  Adapt as necessary
xlower = 0; xupper = 100;
ylower = 0; yupper = 10;
bounds = [xlower xupper ylower yupper];

%Setting the default values for parameters if none are input by user
if (nargin >=1 && (mod(samp_size,1) || samp_size <0))
    error(sprintf(['Correct syntax is CLT_Random(samp_size, num_samp, statistic).' ...
        '\nInvalid entry for input parameter "samp_size".  samp_size must be a positive integer.']))
end

if (nargin >=2 && (mod(num_samp,1) || num_samp <0))
    error(sprintf(['Correct syntax is CLT_Random(samp_size, num_samp, statistic).' ...
        '\nInvalid entry for input parameter "num_samp".  num_samp must be a positive integer.']))
end
if (nargin ==3 && ~(strcmp(statistic, 'mean') || strcmp(statistic, 'median') || strcmp(statistic, 'stdev')))
    error(sprintf(['Correct syntax is CLT_Random(samp_size, num_samp, statistic).' ...
        '\nInvalid entry for input parameter "statistic".'...
        '\nPossible values for statistic are ''mean'', ''median'', or ''stdev'' ']))
end
if (nargin<3)
    statistic = 'mean';
end
if (nargin <2)
    num_samp = 500;
end
if (nargin ==0)
    samp_size = 50;
end


err = 1;        % Flag indicating whether or not a valid probability
                % density function has been created.

% Continue to ask for input until a valid density function has been created.
while (err ==1)
    clf;
    [x, y, err] = ginput_ER(bounds);
end

if (err ==2)
    return;      % User has chosen to exit the program
end


% Plot the population distribution on the top half of the figure.  Sampling
% distribution is plotted on the bottom half of the figure.
figure(2); subplot(2,1,1); plot(x,y)
axis(bounds);
set(gca,'fontsize',12)
X=xlower:(xupper-xlower)/100:xupper;    % Used to create a histogram with 100 bars
title('Population Distribution', 'fontsize', 12);

% Initialize the vector holding statistic of interest for each sample from
% the population
x_bar = zeros(1,num_samp);

Y_upper = 75;   % Axis bound for sampling distribution

% Draw our samples and plot the histrogram of the sampling distribution
for i = 1:num_samp
    % Use the rejection sampler to draw a sample from the population
    samp = rejection_sampler(x,y, samp_size);
    
    if (strcmp(statistic, 'mean'))
        x_bar(i) = mean(samp);
    elseif (strcmp(statistic, 'median'))
        x_bar(i) = median(samp);
    else
        x_bar(i) = std(samp);
    end
    
    % Plot the sampling distribution.  Histogram is updated with each sampled value
    subplot(2,1,2);
    counts = hist(x_bar(1:i),X);    % Counts specifies height of histogram
    if (max(counts) > Y_upper)
        Y_upper = 2*Y_upper;        % Adjust the axis if histogram gets too tall
    end
    
    hist(x_bar(1:i),X);
    if( strcmp(statistic, 'mean') || strcmp(statistic, 'median') )    
        axis([xlower xupper 0 Y_upper]);
        % Note: For proper scaling, the y-limits may need to be changed 
    else
        axis([0 xupper/2 0 Y_upper]);
    end
    title('Drawing Samples from the Population', 'fontsize', 12);
    ylabel('Frequency', 'fontsize', 12);
    set(gca,'fontsize',12)
    pause(0.025);       % A pause allows you to watch as the histogram grows
    
end


% Now let's determine the mean / median / standard deviation 
% of the entire population to use as a comparison with the
% mean / median / standard deviation of the sampling distribution.
%
% Use 10,000 sampled values to get a good approximation of the true
% population distribution

samp = rejection_sampler(x,y, 10000);

disp('Mean of the Population:');
disp(mean(samp));
disp('Median of the Population:');
disp(median(samp));
disp('Standard Deviation of the Population:');
disp(std(samp));


disp('Sample Size:')
disp(samp_size);
disp('Mean of Sampling Distribution:')
disp(mean(x_bar));
disp('Median of Sampling Distribution:')
disp(median(x_bar));
disp('Standard Deviation of Sampling Distribution:')
disp(std(x_bar));
